% Copyright  2012 Computational Biomedicine Lab (CBL), 
% University of Houston. All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, is prohibited without the prior written consent of CBL.
%
% This script allows the operator to manually choose parameters to generate
% training indices, in search for a suboptimal set.
% 

% CLEAR MEMORY
clc; clear;

%%%%%%%%%%%%%%%%%%%% 
%%%  EXPERIMENT  %%% 
%%%%%%%%%%%%%%%%%%%% 

%%%%%%%%%%%%%%%%%%%% 
% CHANGING OPTIONS % 
%%%%%%%%%%%%%%%%%%%% 

%%%%%%%%% 
% model % 
%%%%%%%%% 
daClock = clock;
modelPath ='E:\Model'; % Change accordingly.
modelName = 'diademOP';
model.suffix = '_dOP';
maxNoise = 27;

%%%%%%% 
% exp % 
%%%%%%% 
exp.genLap = false;
% exp.genLap = true;
exp.lapFact = [0.25; 0.5; 0.75];

%%% Olfactory Projection Fibers
exp.boxes = {[300 450 125 250 4 52; 127 296 203 315 25 60; 26 81 243 361 5 41]};
exp.pos = {[1.4 1.1;1 1.1;1.25 1.1]};
exp.pathList = {'C:\Users\david\Desktop\for David\Raw\OP\Olfactory_Fibers_OP_1'};
exp.nameList = {'Olfactory_Fibers_OP_1'}; 

exp.featType = {'Lap','LP','HP'};
exp.featParam = {[.25; .5; .75], [.3; .5; .7], [.6 .3; .4 .1; .8 .5]}; 

exp.spBySize = true; 
exp.spByQuan = false; 
exp.prtSiz = 2500; 
% exp.prtSiz = 10000; 
exp.numPrt = 0; 
exp.numExp = 1000; 

%%%%%%%%%%%%%%%%% 
% FIXED OPTIONS % 
%%%%%%%%%%%%%%%%% 

savMod = fullfile(modelPath,modelName);
exp.genLap = true;
exp.trainIdx = true; 
exp.predIdx = false; 
exp.saveLap = true;
exp.saveTrain = true;

%%%%%%%%%%% 
% USELESS % 
%%%%%%%%%%% 
exp.predStr = '';
exp.predPerc = 0; 
exp.eps = 1; 
exp.binNameList = {''};

%%%%%%%%%%%%% 
%  RUNNING  % 
%%%%%%%%%%%%% 
tic; exp0 = genIdx(exp);        toc; daClock = [clock; daClock];
tic; exp0 = genExperiment(exp0); toc; daClock = [clock; daClock];
tic; exp0 = runExperiment(exp0); toc; daClock = [clock; daClock];

%%%%%%%%%%%%%% 
% ZERO MODEL % 
%%%%%%%%%%%%%% 
model.g0 = exp0.bestG(1); model.c0 = exp0.bestC(1);
model.oS0 = ['-b 1 -c ' num2str(model.c0) ' -g ' num2str(model.g0)];
model.bounds0 = exp0.bounds;
model.featType0 = exp0.featType;
model.featParam0 = exp0.featParam;
tic; model.zero = svmtrain(double(cat(1,exp0.labels{:})),...
    double(cat(1,exp0.feats{:})), model.oS0); toc;
daClock = [clock; daClock];

%%%%%%%%%%%%%%%%%% 
% IDENTIFY NOISE % 
%%%%%%%%%%%%%%%%%% 
for k = 1:length(exp0.nameList)
    R = RAWfromMHD(exp0.nameList{k},[],exp0.pathList{k});
    mm = mean(squeeze(mean(squeeze(mean(R)))));
    I = find(R > mm);
    F = genFeats(R,I,exp);
    for n = 1:size(F,2)
        M = model.bounds0(1,n);
        m = model.bounds0(2,n);
        V = F(:,n);
        V = (V-m)/(M-m);
        F(:,n) = V;
    end
    [L a P] = svmpredict(I,double(F),model.zero,'-b 1');
    B = zeros(size(R),'uint8');
    B(I) = L;
    S = connComp(B);
    [D Idx] = viablePairs(S);
    A = zeros(S.compNum);
    A(Idx) = D;
    A = A+A';
    AA = sum(A,2);
    J = find((AA == 0)&(S.compCard < maxNoise));
    II = cat(1,S.compIdx{J});
    exp.nTrainIdx{k} = cat(1,exp0.nTrainIdx{k},II);
    exp.pTrainIdx{k} = setdiff(exp0.pTrainIdx{k},II);
end

%%%%%%%%%%%%% 
%  RUNNING  % 
%%%%%%%%%%%%% 
tic; exp = genExperiment(exp); toc; daClock = [clock; daClock];
tic; exp = runExperiment(exp); toc; daClock = [clock; daClock];

%%%%%%%%% 
% MODEL % 
%%%%%%%%% 
model.g = exp.bestG(1); 
model.c = exp.bestC(1);
model.oS = ['-b 1 -c ' num2str(model.c) ' -g ' num2str(model.g)];
model.bounds = exp.bounds;
model.featType = exp.featType;
model.featParam = exp.featParam;
tic; model.svm = svmtrain(double(cat(1,exp.labels{:})),...
    double(cat(1,exp.feats{:})), model.oS); toc;
daClock = [clock; daClock];
save(savMod,'model'); daClock = [clock; daClock];

% CREATED: 
% - Date: 09/06/2012
% - By: David Jimenez.
% 
